@@ -1,5 +1,5 @@
 /*****************************************************************************
- * Copyright (C) 2013 x265 project
+ * Copyright (C) 2013-2017 MulticoreWare, Inc
  *
  * Authors: Steve Borho <steve@borho.org>
  *
@@ -131,7 +131,11 @@ YUVInput::YUVInput(InputFileInfo& info)
     if (info.skipFrames)
     {
 #if X86_64
-        ifs->seekg((uint64_t)framesize * info.skipFrames, ios::cur);
+        if (ifs != &cin)
+            ifs->seekg((uint64_t)framesize * info.skipFrames, ios::cur);
+        else
+            for (int i = 0; i < info.skipFrames; i++)
+                ifs->read(buf[0], framesize);
 #else
         for (int i = 0; i < info.skipFrames; i++)
             ifs->ignore(framesize);
